#ifndef GICID_BACKEND_H
#define	GICID_BACKEND_H

/*
 * $Id: backend.h,v 1.4 2001/03/04 01:08:56 andy Exp $
 *
 * Copyright  2000, A.Newman.
 * All Rights Reserved.
 */

/*
 * Name of the backend executable.  This MUST obey protocol
 * as outlined in icid/PROTOCOL.
 *
 */
#define	GICID_CMD	"/usr/local/lib/ici3/gicid"


/*
 * Backend process states.
 */
enum
{
    Starting,			/* Child is starting up */
    Stopped,			/* Child is stopped awaiting commands */
    Running,			/* Child is running */
    Breakpoint,			/* Child hit breakpoint */
    Watchpoint,			/* Child hit watchpoint */
    Finished,			/* Child has finished execution */
    Errored,			/* Child raised error */
    Broken			/* Child is broken is some way */
};


/*
 * Backend instance structure.
 *
 * A backend is a process running an instance of gicid.  The backend
 * is attached to this process via two pipes, one used to receive
 * state output from the program, the other used to issue requests.
 */
typedef struct Backend	Backend;
struct Backend
{
    int		tofd;			/* Command pipe */
    int		fromfd;			/* Event/response pipe */
    FILE	*fromf;			/* And stream layered atop that */
    int		pid;			/* Child's pid */
    int		state;			/* Current state of child */
    int		err;			/* Errno iff state == Broken */
    char	*file;			/* Last file name from source marker */
    int		line;			/* Last line number from source marker */
    char	*icierr;		/* ICI error iff state == Error */
    int		inputid;		/* Gtk input handler id. */
    int		status;			/* Exit status iff state == Finished */
    void	(*cb)(Backend *);	/* Callback when state changes */
    char	buf[1024];		/* Input buffer for child */
};


/*
 * Create a new backend process with the given argv.
 */
Backend		*Backend_new(char **);

/*
 * Destroy the backend process and data structure, freeing
 * all resources (memory, pipes, process).
 */
void		Backend_destroy(Backend *);

/*
 * Read status output from the backend process and sychronise
 * our view of the process state. I.e, interpret the status side
 * of the icid protocol.
 */
int		Backend_sync(Backend *);

/*
 * Have the backend perform a single-step
 */
int		Backend_step(Backend *);

/*
 * Have the backend single step but step over function calls.
 */
int		Backend_next(Backend *);

/*
 * Have the backend return from the currently called function.
 */
int		Backend_finish(Backend *);

/*
 * Have the backend continue execution of the user program.
 */
int		Backend_continue(Backend *);

/*
 * Interrupt a running backend and force it to reply
 * to the UI.
 */
int		Backend_interrupt(Backend *);

/*
 * Listen for output from the backend in the Gtk event loop
 * and handle status changes. Call supplied function when
 * backend sends status.
 */
int		Backend_listen(Backend *, void (*cb)(Backend *));


int		Backend_where(Backend *, char ***, int *);
void		Backend_free_where(char **, int);

void		*Backend_get_scope(Backend *);
int		Backend_set_scope(Backend *, void *);
int		Backend_scope_up(Backend *);
int		Backend_fetch(Backend *, const char *, char *, int);
int		Backend_assign(Backend *, const char *, const char *);
int		Backend_scope_keys(Backend *, char ***, int *);

#endif /* #ifndef GICID_BACKEND_H */

/*
 * $Log: backend.h,v $
 * Revision 1.4  2001/03/04 01:08:56  andy
 * Scope manipulation added.
 *
 * Revision 1.3  2001/02/26 20:53:25  andy
 * Commentary.
 *
 * Revision 1.2  2001/02/24 23:02:26  andy
 * Pick up protocol specific values from gicid header and use them.
 * Modify send_cmd() to suit enum-defined command codes.
 *
 */
