#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "cachedfile.h"

char	*icipath = NULL;

CachedFile *
CachedFile_new(const char *name)
{
    CachedFile	*cf;

    if ((cf = malloc(sizeof (CachedFile) + strlen(name) + 1)) == NULL)
	return NULL;
    cf->name = strcpy((char *)(cf + 1), name);
    cf->lines = NULL;
    cf->nlines = 0;
    cf->maxlines = 0;
    cf->size = 0;
    return cf;
}

void
CachedFile_destroy(CachedFile *cf)
{
    if (cf->lines)
    {
	int	i;
	
	for (i = 0; i < cf->nlines; ++i)
	    free(cf->lines[i]);
	free(cf->lines);
    }
    free(cf);
}

static int
growlines(CachedFile *cf)
{
    int		newmax;
    char	**newlines;

    if (cf->lines == NULL)
    {
	newmax = 256;
	if ((newlines = malloc(newmax * sizeof (char *))) == NULL)
	    return 1;
    }
    else
    {
	if (cf->maxlines < 65536)
	    newmax = cf->maxlines * 2;
	else
	    newmax = cf->maxlines * 1.125;
	if ((newlines = realloc(cf->lines, newmax * sizeof (char *))) == NULL)
	    return 1;
    }
    cf->lines = newlines;
    cf->maxlines = newmax;
    return 0;
}

static int
addline(CachedFile *cf, const char *line)
{
    char	*p;

    if ((cf->nlines == cf->maxlines && growlines(cf)) || (p = strdup(line)) == NULL)
	return 1;
    cf->lines[cf->nlines++] = p;
    cf->size += strlen(p);
    return 0;
}

static FILE *
pfopen(const char *fname)
{
    FILE	*file;
    char	*p;

    if ((file = fopen(fname, "r")) != NULL)
	return file;
    if (icipath == NULL)
    {
	if ((icipath = getenv("ICIPATH")) == NULL)
	    icipath = ".:/usr/local/lib/ici3";
    }
    p = icipath;
    for (;;)
    {
	char	path[FILENAME_MAX];
	char	*e;

	if ((e = strchr(p, ':')) == NULL)
	    e = p + strlen(p);
	sprintf(path, "%.*s/%s", e - p, p, fname);
	if ((file = fopen(path, "r")) != NULL)
	    return file;
	if (*(p = e + 1) == 0)
	    break;
    }
    return NULL;
}

int
CachedFile_read(CachedFile *cf)
{
    FILE	*file;
    char	buffer[4096];

    if ((file = pfopen(cf->name)) == NULL)
	return 1;
    while (fgets(buffer, sizeof buffer, file))
	addline(cf, buffer);
    fclose(file);
    return 0;
}

char *
CachedFile_line(CachedFile *cf, int n)
{
    if (n < 1 || n > cf->nlines)
	return NULL;
    return cf->lines[n-1];
}

unsigned int
CachedFile_linepos(CachedFile *cf, int n)
{
    unsigned int	pos;
    int			i;

    if (n <= 1)
	return 0;
    if (n > cf->nlines)
	return cf->size;
    for (pos = 0, i = 0; i < n-1; ++i)
	pos += strlen(cf->lines[i]);
    return pos;
}
