#ifndef GICID_FILECACHE_H
#define	GICID_FILECACHE_H

/*
 * A CachedFile is a text file that is stored in memory. Each cached file
 * is identified by name and stores each line of text as a separately
 * allocated string.  The overall index of lines is itself allocated
 * dynamically and grows as required.
 */
typedef struct
{
    char	*name;
    char	**lines;
    int		nlines;
    int		maxlines;
    int		size;
}
CachedFile;

/*
 * Create a new CachedFile object representing the named file.
 */
CachedFile	*CachedFile_new(const char *name);

/*
 * Read the file into memory and create the line index.
 */
int		CachedFile_read(CachedFile *);

/*
 * Return line N from the file. The first line is 1.
 */
char		*CachedFile_line(CachedFile *, int);

/*
 * Destroy a cached file, freeing any memory used
 * by the object.
 */
void		CachedFile_destroy(CachedFile *);

/*
 * Return the aggregate character position of the start
 * of line N.
 */
unsigned int	CachedFile_linepos(CachedFile *, int);

#endif /* #ifndef GICID_FILECACHE_H */
