#include <stdio.h>

#include "cachedfile.h"
#include "filecache.h"

enum
{
    MAXFILES = 50
};

static CachedFile	*cache[MAXFILES];
static int		refs[MAXFILES];

static CachedFile **
findslot(void)
{
    int		i;
    int		fi;
    int		f;

    for (fi = -1, f = 0x7fffffff, i = 0; i < MAXFILES; ++i)
    {
	if (cache[i] == NULL)
	{
	    refs[i] = 0;
	    return &cache[i];
	}
	if (refs[i] < f)
	{
	    f = refs[i];
	    fi = i;
	}
    }
    if (fi != -1)
    {
	CachedFile_destroy(cache[fi]);
	refs[fi] = 0;
	return &cache[fi];
    }
    return NULL;
}

static CachedFile **
findfile(const char *fname)
{
    int	i;

    for (i = 0; i < MAXFILES; ++i)
	if (cache[i] != NULL && strcmp(fname, cache[i]->name) == 0)
	    return &cache[i];
    return NULL;
}

int
FileCache_open(const char *name)
{
    CachedFile	*cf;
    CachedFile	**cfp;

    if ((cfp = findfile(name)) != NULL)
	return cfp - cache;
    if ((cf = CachedFile_new(name)) == NULL)
	return -1;
    if (CachedFile_read(cf))
    {
	CachedFile_destroy(cf);
	return -1;
    }
    if ((cfp = findslot()) == NULL)
	return -1;
    *cfp = cf;
    return cfp - cache;
}

int
FileCache_close(int fno)
{
    if (fno < 0 || fno > MAXFILES || cache[fno] == NULL)
	return 1;
    CachedFile_destroy(cache[fno]);
    cache[fno] = NULL;
    refs[fno] = 0;
    return 0;
}

char *
FileCache_line(int fno, int lno)
{
    if (fno < 0 || fno > MAXFILES || cache[fno] == NULL)
	return NULL;
    return CachedFile_line(cache[fno], lno);
}

int
FileCache_nlines(int fno)
{
    if (fno < 0 || fno > MAXFILES || cache[fno] == NULL)
	return NULL;
    return cache[fno]->nlines;
}


unsigned int
FileCache_linepos(int fno, int lno)
{
    if (fno < 0 || fno > MAXFILES || cache[fno] == NULL)
	return NULL;
    return CachedFile_linepos(cache[fno], lno);
}
